# -*- coding: utf-8 -*-
from setuphelpers import *


appx_package_name = ""
main_appx_package_name = ""
appx_dir = makepath(programfiles, "WindowsAppsInstallers")
remove_sources = False  # remove_sources = True may prevent multi-user installations.


def install():
    # Declare local variables
    bins_dir = control.package.split("-", 1)[1]
    appx_bins_dir = makepath(appx_dir, bins_dir)

    # Remove from old path
    for appx_file in glob.glob(makepath(appx_dir, "%s_*" % appx_package_name)):
        remove_file(appx_file)

    # Placing bins for session-setup installs
    if isdir(appx_bins_dir):
        remove_tree(appx_bins_dir)
    mkdirs(appx_bins_dir)
    print(f"Creating: {appx_bins_dir}")
    for f in glob.glob(bins_dir + "/*"):
        filecopyto(f, appx_bins_dir)
    for f in glob.glob(appx_bins_dir + "/*"):
        fname = f.split(os.sep)[-1]
        if control.architecture == "all":
            if "x86" in glob.glob(makepath(appx_bins_dir, f"{appx_package_name}_*"))[0].split(os.sep)[-1]:
                if not "x86" in fname and not "neutral" in fname:
                    remove_file(f)
            else:
                if not get_host_architecture() in fname and not "neutral" in fname:
                    remove_file(f)
        else:
            if not get_host_architecture() in fname and not "neutral" in fname:
                remove_file(f)


def session_setup():
    # Declare local variables
    bins_dir = control.package.split("-", 1)[1]
    appx_bins_dir = makepath(appx_dir, bins_dir)
    try:
        bin_path = glob.glob(makepath(appx_bins_dir, f"{appx_package_name}_*"))[0]
    except:
        error("Unable to find sources.\nPlease reinstall this package or change remove_sources to False if relevant.")
    dependencies_pathes = ",".join([f'"{a}"' for a in glob.glob(makepath(appx_bins_dir, "*")) if not appx_package_name in a])
    add_appx_cmd = f'Add-AppxPackage -Path "{bin_path}"'
    if dependencies_pathes:
        add_appx_cmd += f" -DependencyPath {dependencies_pathes}"

    # Installing the UWP application in the user environment if needed
    if main_appx_package_name:
        main_appx_package = run_powershell(f'Get-AppxPackage -Name "{main_appx_package_name}"')
        appx_dependencies = None if main_appx_package is None else main_appx_package.get("Dependencies", None)
        appx_dependency = None
        for appx in appx_dependencies:
            if not appx["IsResourcePackage"] and appx["Name"] == appx_package_name:
                appx_dependency = appx
                break
        appx_version = None if appx_dependency is None else appx_dependency.get("Version", None)
    else:
        appx_package = run_powershell(f'Get-AppxPackage -Name "{appx_package_name}"')
        appx_version = None if appx_package is None else appx_package.get("Version", None)

    if Version(appx_version) < Version(control.get_software_version()) or force:
        print(f"Installing: {bin_path.split(os.sep)[-1]}")
        killalltasks(ensure_list(control.impacted_process))
        run_powershell(add_appx_cmd, output_format="text")
    else:
        print(f"{appx_package_name} ({appx_version}) is installed and up-to-date")


def uninstall():
    # Declare local variables
    bins_dir = control.package.split("-", 1)[1]
    appx_bins_dir = makepath(appx_dir, bins_dir)

    if main_appx_package_name:
        print(f'You must uninstall: "{main_appx_package_name}" to remove this dependency.')
    else:
        print(f'You must uninstall: the "main_appx_package_name" (not specified) to remove this dependency.')

    if isdir(appx_bins_dir):
        print("Removing: %s" % (appx_bins_dir))
        remove_tree(appx_bins_dir)
    if dir_is_empty(appx_dir):
        print("Removing: %s since it is empty" % (appx_dir))
        remove_tree(appx_dir)
